=begin
 /****************************************************************
**
** Implementation CannonField class, Qt tutorial 8
**
****************************************************************/
=end
# For rubyQt
require "qt"
include Qt
include Math

class CannonField < QWidget
def initialize(parent=nil,name="")
   @angleChanged = RSignal.new("int")  # signal with int
   @forceChanged = RSignal.new("int")
   @hit = RSignal.new("")
   @ang = 45
   @f = 0
   @shooting = false
   @timerCount = 0
   @shoot_ang = 0
   @shoot_f = 0
   @target = QPoint.new(0,0)
   @barrel_rect = QRect.new(33,-4, 15, 8)
   @bcolor = QColor.new(0,0,0)
   @first_time = true
   newTarget
   catchEvent
end

def angleChanged
   @angleChanged
end
def forceChanged
   @forceChanged
end
def hit
   @hit
end
def setAngle degrees #slot
   if degrees < 5
      degrees = 5
   elsif degrees > 70
      degrees = 70
   elsif degrees == @ang
      return
   end
   @ang = degrees
   repaintQRect(cannonRect,false)
   angleChanged.send_int(@ang)
end

def setForce newton
   if newton < 0
      newton = 0
   elsif @f == newton
      return
   end
   @f = newton
   forceChanged.send_int(@f)
end

def shoot
   if (@shooting)
      return
   end
   @timerCount = 0
   @shoot_ang = @ang
   @shoot_f = @f
   @shooting = true
   startTimer(50)
end


def newTarget

   if( @first_time )
      @first_time = false
      midnight = QTime.new( 0, 0, 0)
       srand (midnight.secsTo(QTime::currentTime))
   end
   erase(targetRect)
   @target = QPoint.new( 200 + rand(6400) % 190, 10 + rand(6400)  % 255)
   repaintQRect(targetRect)
end

def timerEvent ev
   erase(shotRect)
   @timerCount += 1;
   shotR = shotRect
   if (shotR.intersects( targetRect))
      stopShooting
      @hit.send
      return
   end
   if (shotR.x > width || shotR.y > height)
      stopShooting
      return
   end

   repaintQRect( shotR, false)
end

def paintEvent ev
    updateR = ev.rect
    p = QPainter.new(self)
    if (updateR.intersects( cannonRect ))
        paintCannon( p )
    end
    if (@shooting && updateR.intersects( shotRect))
        paintShot( p )
    elsif (updateR.intersects(targetRect))
       paintTarget(p)
    end
    p.end
end

def stopShooting
   @shooting = false
   killTimers
end

def paintShot p
   brush = QBrush.new(QColor.new(0,0,0))
   pen = QPen.new(QColor.new(0,0,0))
   pen.setStyle(NoPen)
   p.setBrush(brush)
   p.setPen(pen)
   r = shotRect
   p.drawRect(r.x, r.y, r.width, r.height)
end

def paintTarget p
   brush = QBrush.new(QColor.new(255,0,0))
   pen = QPen.new(QColor.new(0,0,0))
   p.setBrush(brush)
   p.setPen(pen)
   p.drawRect(targetRect.x ,targetRect.y,targetRect.width,targetRect.height)
end

def paintCannon p
   cr = cannonRect
   pix = QPixmap.new(QSize.new(cr.width, cr.height))
   pix.fill(@bcolor)
   tmp = QPainter.new(pix)
   brush = QBrush.new(QColor.new(0,0,255))
   pen = QPen.new( QColor.new(0,0,0))
   pen.setStyle(NoPen)

   tmp.setBrush(brush)
   tmp.setPen(pen)

   tmp.translate(0, pix.height - 1)
   tmp.drawPie( -35,-35,70,70,0,90*16)
   tmp.rotate( - @ang )
   tmp.drawRect( @barrel_rect.x, @barrel_rect.y,
        @barrel_rect.width, @barrel_rect.height)
   tmp.end

   p.drawPixmap( cr.x,cr.y,pix)
end

def cannonRect
    r = QRect.new(0, 0, 50, 50)
    r.moveBottomLeft(QPoint.new(0,self.height))
    r
end

def  setBackgroundColor qcolor
   @bcolor = qcolor
   super
end

def shotRect
   gravity = 4.0

   time = @timerCount / 4.0
   velocity = @shoot_f
   radians = @shoot_ang * PI/180

   velx = velocity * cos(radians)
   vely = velocity * sin(radians)
   x0 = (@barrel_rect.right + 5) * cos(radians)
   y0 = (@barrel_rect.right + 5) * sin(radians)
   x = x0 + velx * time
   y = y0 + vely * time - 0.5 * gravity * time * time

   r = QRect.new(0,0,6,6)
   r.moveCenter(QPoint.new(x.round,height - 1 - y.round))
   r
end

def targetRect
   r = QRect.new(0,0,20,10)
   r.moveCenter(QPoint.new(@target.x,height - 1 - @target.y))
   r
end

end
