=begin
/****************************************************************
**
** Implementation of GameBoard class, Qt tutorial 13
**
****************************************************************/
=end
# For rubyQt 20000213 by N. Horie

require "qt"
include Qt
require "lcdrange.rb"
require "cannon.rb"

class GameBoard < QWidget
def initialize

    setMinimumSize( 500, 355 );

    @quit = QPushButton.new( "Quit", self, "quit" )
    @quit.setFont( QFont.new( "Times", 18, QFont::Bold ) )

    Qt::connect( @quit, QSIGNAL("clicked()"),
              QApplication::qApp, QSLOT("quit()") )

    @angle  = LCDRange.new( self, "angle" )
    @angle.setText("ANGLE")
    @angle.setRange( 5, 70 )

    @force  = LCDRange.new( self, "force" )
    @force.setText("FORCE")
    @force.setRange( 10, 50 )

    @cannonField = CannonField.new( self, "cannonField" )
    @cannonField.setBackgroundColor( QColor.new( 250, 250, 200) )

    Qt::connect( @angle.valueChanged, @cannonField,"setAngle")
    Qt::connect( @cannonField.angleChanged, @angle,"setValue")

    Qt::connect( @force.valueChanged, @cannonField,"setForce")
    Qt::connect( @cannonField.forceChanged, @force,"setValue")

    Qt::connect( @cannonField.hit,self,"hit" )
    Qt::connect( @cannonField.missed,self,"missed" )

    @angle.setValue( 60 )
    @force.setValue( 25 )

    @shoot = QPushButton.new( "Shoot", self, "shoot" )
    @shoot.setFont( QFont.new( "Times", 18, QFont::Bold ) )

    Qt::connect( @shoot, QSIGNAL("clicked()"), self,"fire" )

    @restart = QPushButton.new( "New Game", self, "newgame" )
    @restart.setFont( QFont.new( "Times", 18, QFont::Bold ) )

    Qt::connect( @restart, QSIGNAL("clicked()"), self,"newGame" )

    @hits  	       = QLCDNumber.new( 2, self, "hits" )
    @shotsLeft 	       = QLCDNumber.new( 2, self, "shotsleft" );
    hitsL      = QLabel.new( "HITS", self, "hitsLabel" );
    shotsLeftL = QLabel.new( "SHOTS LEFT", self, "shotsleftLabel" )

    @quit.setGeometry( 10, 10, 75, 30 );
    @angle.setGeometry( 10, @quit.y + @quit.height + 10, 75, 130 )
    @force.setGeometry( 10, @angle.y + @angle.height + 10, 75, 130 )
    @cannonField.move( @angle.x + @angle.width + 10, @angle.y )
    @shoot.setGeometry( 10, 315, 75, 30 )
    @restart.setGeometry( 380, 10, 110, 30 )
    @hits.setGeometry( 130, 10, 40, 30 )
    hitsL.setGeometry( @hits.x + @hits.width + 5, 10, 60, 30 )
    @shotsLeft.setGeometry( 240, 10, 40, 30 )
    shotsLeftL.setGeometry( @shotsLeft.x + @shotsLeft.width + 5, 10, 60, 30 )

    newGame
    catchEvent
end


def resizeEvent e

    @cannonField.resize( width  - @cannonField.x - 10,
			 height - @cannonField.y - 10 )
end

def fire

    if ( @cannonField.gameOver or @cannonField.isShooting )
	return
    end
    @shotsLeft.display( @shotsLeft.intValue() - 1 );
    @cannonField.shoot
end

def hit

    @hits.display( @hits.intValue + 1 );
print  @shotsLeft.intValue ; print "\n"
    if ( @shotsLeft.intValue == 0 )
	@cannonField.setGameOver
    else
	@cannonField.newTarget
    end
end

def missed
print  @shotsLeft.intValue ; print "\n"
    if ( @shotsLeft.intValue == 0 )
	@cannonField.setGameOver
    end
end

def newGame

    @shotsLeft.display( 15 )
    @hits.display( 0 )
    @cannonField.restartGame
    @cannonField.newTarget
end
end
