=begin
/****************************************************************************
** $Id: application.cpp,v 1.16.2.1 1999/02/01 19:58:21 hanord Exp $
**
** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
**
** This file is part of an example program for Qt.  This example
** program may be used, distributed and modified without limitation.
**
*****************************************************************************/
=end

require "qt"
include Qt


class ApplicationWindow < QMainWindow
    NewDoc = 1
    Open = 2
    Save = 3
    Print = 4
    Close = 5
    ToggleMenuBar = 6
    ToggleToolBar = 7
    ToggleStatusBar = 8
    MARGIN = 10
    FileOpenText = "Click this button to open a new file.\n\nYou can also select the Open command from the File menu."
    FileSaveText = "Click this button to save the file you are editing.  You will be prompted for a file name.\n\nYou can also select the Save command from the File menu.\n\nNote that implementing this function is left as an exercise for the reader.";
    FilePrintText = "Click this button to print the file you are editing.\n\nYou can also select the Print command from the File menu.";


def initialize
    super 0, "example application main window"
    @printer = QPrinter.new
    @printer.setMinMax( 1, 10 );
    @openIcon = QPixmap.new("fileopen.xpm")
    @saveIcon = QPixmap.new("filesave.xpm")
    @printIcon = QPixmap.new("fileprint.xpm")
    @fileTools = QToolBar.new( self, "file operations" )

    @fileOpen = QToolButton.new( @openIcon, "Open File", 0,
					    self, "load",
					    @fileTools, "open file" )

    @fileSave = QToolButton.new( @saveIcon, "Save File", 0,
					      self, "save",
					      @fileTools, "save file" );

    @filePrint = QToolButton.new( @printIcon, "Print File", 0,
					       self, "printIt",
					       @fileTools, "print file" );
    @fileTools.addSeparator
    QWhatsThis::whatsThisButton( @fileTools )
    QWhatsThis::add( @fileOpen, FileOpenText, FALSE )
    QWhatsThis::add( @fileSave, FileSaveText, FALSE )
    QWhatsThis::add( @filePrint, FilePrintText, FALSE )
    
    @file = QPopupMenu.new
#    @menuBar = self.menuBar
    menuBar.insertItem( "&File", @file )
    Qt::connect(menuBar,QSIGNAL("activated(int)"),self,"menuActivated")

#    @file.insertItem( "New", self, "newDoc", Qt::CTRL+Key_N );
    @file.insertItem("New",NewDoc,-1)
    @file.setAccel(NewDoc,Qt::CTRL+Key_N)

#    @file.insertItem( openIcon, "Open", self, SLOT(load()), CTRL+Key_O );
    @file.insertItemQPixmapTextId(@openIcon,"Open",Open,-1)
    @file.setAccel(Open,Qt::CTRL+Key_O)

#    @file.insertItemQPixmapTextId( saveIcon, "Save", this, SLOT(save()), CTRL+Key_S );
    @file.insertItemQPixmapTextId(@saveIcon,"Save",Save,-1)
    @file.setAccel(Save,Qt::CTRL+Key_S)

    @file.insertSeparator

#    file->insertItem( printIcon, "Print", this, SLOT(print()), CTRL+Key_P );
    @file.insertItemQPixmapTextId(@printIcon,"Print",Print,-1)
    @file.setAccel(Print,Qt::CTRL+Key_P)

    @file.insertSeparator

#    file->insertItem( "Close", this, SLOT(closeDoc()), CTRL+Key_W );
    @file.insertItem("Close",Close,-1)
    @file.setAccel(Close,Qt::CTRL+Key_W)

#    file->insertItem( "Quit", qApp, SLOT(quit()), CTRL+Key_Q );
#    @file.insetItemQObj("Quit",QApplication::qApp,QSLOT("quit()"),Qt::CTRL+Key_Q)

    @controls = QPopupMenu.new
    menuBar.insertItem( "&Controls", @controls )

    @mb = @controls.insertItem( "Menu bar", ToggleMenuBar,-1)
    @controls.setAccel(ToggleMenuBar,Qt::CTRL+Key_M)
    #// Now an accelerator for when the menubar is invisible!
    @a = QAccel.new(self)
    @a.connectItem( @a.insertItem( CTRL+Key_M ), self, "toggleMenuBar" )

    @tb = @controls.insertItem( "Tool bar", ToggleToolBar,-1)
    @controls.setAccel(ToggleToolBar, Qt::CTRL+Key_T)

    @sb = @controls.insertItem( "Status bar", ToggleStatusBar, -1)
    @controls.setAccel(ToggleStatusBar, Qt::CTRL+Key_B)
    @controls.setCheckable( true )
    @controls.setItemChecked( @mb, true )
    @controls.setItemChecked( @tb, true )
    @controls.setItemChecked( @sb, true )

    @e = QMultiLineEdit.new( self, "editor" )
    @e.setFocus
    setCentralWidget( @e );
    self.statusBar.message( "Ready", 2000 )
end

def menuActivated id
   case id
   when NewDoc
      newDoc
   when Open
      load
   when Save
      save
   when Print
      printIt
   when Close
      closeDoc
   when ToggleMenuBar
      toggleMenuBar
   when ToggleToolBar
      toggleToolBar
   when ToggleStatusBar
      toggleStatusBar
   end
end



def newDoc
    ed = ApplicationWindow.new
    ed.resize( 400, 400 )
    ed.show
end

def load

    fn = QFileDialog::getOpenFileName("","*.txt",self)
    if ( !fn.isEmpty() )
        fname = fn.to_str
	loadFile( fname )
    else
	self.statusBar.message( "Loading aborted", 2000 )
    end
end


def loadFile fn
    @filename = fn
    f = File::open( @filename,"r")

    @e.setAutoUpdate( false );
    @e.clear

    while (!f.eof?)
      s = f.gets
      @e.append(s)
    end
    f.close

    @e.setAutoUpdate( true )
    @e.repaintAll
    setCaption( @filename )

    s =  "Loaded document #{@filename} "
    self.statusBar.message( s, 2000 )
end


def save
    if (@filename == nil )
      @filename = "NewFile"
      qfilename = QFileDialog:: getSaveFileName(@filename,"*.txt",self)
      if( !qfilename.isEmpty )
         @filename = qfilename.to_str
      end
    end
    contents = @e.text
    str = contents.to_str
    f = File::open(@filename,"w+")
    f.print str
    f.close

    s = "Save document to #{@filename} "
    self.statusBar.message(s, 2000)

end


def printIt

    pageNo = 1

    if ( @printer.setup(self) ) 		#// printer dialog
	self.statusBar.message( "Printing..." ,4000)
	p = QPainter.new(nil)
	p.begin( @printer )			#// paint on printer
	p.setFont( @e.font )
	yPos        = 0			 	#// y position for each line
	fm = p.fontMetrics
	metrics = QPaintDeviceMetrics.new( @printer )	 #// need width/height
	                                         	#// of printer surface
	for i in 0..(@e.numLines - 1)
	    if ( MARGIN + yPos > metrics.height - MARGIN )
                pageNo = pageNo + 1
		msg = "Printing (page #{pageNo})..."
		self.statusBar.message( msg ,4000)
		printer.newPage		#// no more room on this page
		yPos = 0			#// back to top of page
	    end
	    p.drawTextXYWH( MARGIN, MARGIN + yPos,
			metrics.width, fm.lineSpacing,
			ExpandTabs || DontClip,
			@e.textLine( i ) )
	    yPos = yPos + fm.lineSpacing
	end
	p.end				#// send job to printer
	self.statusBar.message( "Printing completed", 2000 )
    else
	self.statusBar.message( "Printing aborted", 2000 )
    end

end

def closeDoc

    close( true )	#// close AND DELETE!
end

def toggleMenuBar

    if ( menuBar.isVisible )
	menuBar.hide
	@controls.setItemChecked( @mb, false )
    else
	menuBar.show
	@controls.setItemChecked( @mb, true )
    end
end

def toggleToolBar

    if ( @fileTools.isVisible )
	@fileTools.hide
	@controls.setItemChecked( @tb, false )
    else
	@fileTools.show
	@controls.setItemChecked( @tb, true )
    end
end

def toggleStatusBar

    if ( self.statusBar.isVisible )
	self.statusBar.hide
	@controls.setItemChecked( @sb, false )
    else
	self.statusBar.show
	@controls.setItemChecked( @sb, true )
    end
end

end


