=begin
/****************************************************************************
** $Id: widgets.cpp,v 2.46 1998/07/08 13:22:45 aavit Exp $
**
** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
**
** This file is part of an example program for Qt.  This example
** program may be used, distributed and modified without limitation.
**
*****************************************************************************/
=end
# For rubyQt
require "qt"
include Qt


#// Standard Qt widgets



#// Some sample widgets

#include "../aclock/aclock.h"
#include "../dclock/dclock.h"
require "aclock_lib.rb"
require "dclock_lib.rb"

MOVIEFILENAME = "trolltech.gif"

#//
#// WidgetView contains lots of Qt widgets.
#//

class WidgetView < QWidget
def initialize

  #// Set the window caption/title

    setCaption( "Qt Widgets Demo Application" )

  #// Install an application-global event filter

    #QApplication::qApp.installEventFilter( this );
   catchEvent

  #// Create a layout to position the widgets

    topLayout = QVBoxLayout.new( self, 10 )

  #// Create a grid layout to hold most of the widgets

    grid = QGridLayout.new(self, 6, 3 )  # self OK? or nil?
 #// This layout will get all of the stretch
    topLayout.addLayout( grid, 10 )

  #// Create a menubar
    menubar = QMenuBar.new( self )
    menubar.setSeparator( QMenuBar::InWindowsStyle )
  #// Create an easter egg
    QToolTip::addQRect( menubar, QRect.new( 0, 0, 2, 2 ), "easter egg" )

    popup = QPopupMenu.new
    id = popup.insertItem( "&New" )
    popup.setItemEnabled( id, false )
    id = popup.insertItem( "&Open" )
    popup.setItemEnabled( id, false )
    popup.insertSeparator()
    popup.insertItemQObj( "&Quit", QApplication::qApp, QSLOT("quit()"), CTRL+Key_Q )

    menubar.insertItem( "&File", popup )

    #// Must tell the layout about a menubar in a widget
    topLayout.setMenuBar( menubar )

  #// Create an analog and a digital clock

    @aclock = AnalogClock.new( self )
    @dclock = DigitalClock.new( 0 ,self)

    @dclock.setMaximumWidth(200)
    grid.addWidget( @aclock, 0, 2 )
    grid.addWidget( @dclock, 1, 2 )

  #// Give the dclock widget a blue palette

    col = QColor.new( 0xaa, 0xbe, 0xff )
    @dclock.setPalette( QPalette.new( col ) )

  #// make tool tips for both of them

    QToolTip::add( @aclock, "custom widget: analog clock" );
    QToolTip::add( @dclock, "custom widget: digital clock" );

  #// Create a push button.

    pb = QPushButton.new("Push button 1", self, "button1" )	#// create button 1
#    pb.setText( "Push button 1" )
    pb.setFixedHeight( pb.sizeHint.height )
    grid.addWidget( pb, 0, 0, AlignVCenter )
    Qt::connect( pb, QSIGNAL("clicked()"), self, "button1Clicked" )
    QToolTip::add( pb, "push button 1" )
    pm = QPixmap.new(nil)
    pix = pm.load("qt.bmp");		#// load pixmap for button 2
    if ( !pix )
	QMessageBox::information( 0, "Qt Widgets Example",
				  "Could not load the file \"qt.bmp\", which\n"	+ "contains an icon used...\n\n" + "The text \"line 42\" will be substituted.",
				  QMessageBox::Ok + QMessageBox::Default )
    end

    #// Create a label containing a QMovie

    @movielabel = QLabel.new("", self, "label0" )
    @movie = QRMovie.new( MOVIEFILENAME )
    @movie.connectStatus(self, "movieStatus")
    @movie.connectUpdate(self, "movieUpdate")
    @movielabel.setFrameStyle( QFrame::Box | QFrame::Plain )
    @movielabel.setMovie( @movie )
    @movielabel.setMargin( 0 )
    @movielabel.setFixedSize( 128 + @movielabel.frameWidth * 2,
			      64 + @movielabel.frameWidth * 2 )
    grid.addWidget( @movielabel, 0, 1, AlignCenter )
    QToolTip::add( @movielabel, "movie" )

  #// Create a group of check boxes

    bg = QButtonGroup.new("", self, "checkGroup" )
    bg.setTitle( "Check Boxes" )
    grid.addWidget( bg, 1, 0 )

  #// Create a layout for the check boxes
    vbox = QVBoxLayout.new(bg, 10)

    vbox.addSpacing( bg.fontMetrics.height )
    @cb = []
    @cb[0] = QCheckBox.new("", bg )
    @cb[0].setText( "Read" )
    vbox.addWidget( @cb[0] )
    @cb[0].setMinimumSizeQSize( @cb[0].sizeHint )
    @cb[1] = QCheckBox.new("", bg )
    @cb[1].setText( "Write" )
    vbox.addWidget( @cb[1] )
    @cb[1].setMinimumSizeQSize( @cb[1].sizeHint )
    @cb[2] = QCheckBox.new("", bg )
    @cb[2].setText( "Execute" )
    @cb[2].setMinimumSizeQSize( @cb[2].sizeHint )
    vbox.addWidget( @cb[2] )
    bg.setMinimumSizeQSize( bg.childrenRect.size )
    vbox.activate

    Qt::connect( bg, QSIGNAL("clicked(int)"), self, "checkBoxClicked" )

    QToolTip::add( @cb[0], "check box 1" )
    QToolTip::add( @cb[1], "check box 2" )
    QToolTip::add( @cb[2], "check box 3" )

  #// Create a group of radio buttons

    bg = QButtonGroup.new("", self, "radioGroup" )
    bg.setTitle( "Radio buttons" )

    grid.addWidget( bg, 1, 1 )

  #// Create a layout for the radio buttons
    vbox = QVBoxLayout.new(bg, 10);

    vbox.addSpacing( bg.fontMetrics.height )
    @rb = QRadioButton.new("", bg )
    @rb.setText( "&AM" )
    @rb.setChecked( true )
    vbox.addWidget(@rb)
    @rb.setMinimumSizeQSize( @rb.sizeHint )
    QToolTip::add( @rb, "radio button 1" )
    @rb = QRadioButton.new("", bg );
    @rb.setText( "&FM" )
    vbox.addWidget(@rb)
    @rb.setMinimumSizeQSize( @rb.sizeHint )
    QToolTip::add( @rb, "radio button 2" )
    rb = QRadioButton.new("", bg )
    rb.setText( "&Short Wave" )
    vbox.addWidget(rb)
    rb.setMinimumSizeQSize( rb.sizeHint )
    vbox.activate

    Qt::connect( bg, QSIGNAL("clicked(int)"), self, "radioButtonClicked" )
    QToolTip::add( rb, "radio button 3" )

  #// Create a list box

    lb = QListBox.new( self, "listBox" )
    for i in 0..99 		#// fill list box
	str = "line #{i}"
	if  ( i == 42 && (! pix.nil?) )
	    lb.insertItemQPixmap( pm )
	else
	    lb.insertItem( str )
        end
    end
    grid.addMultiCellWidget( lb, 2, 4, 0, 0 )
    Qt::connect( lb, QSIGNAL("selected(int)"), self, "listBoxItemSelected" )
    QToolTip::add( lb, "list box" )

    vbox = QVBoxLayout.new(self, 8) #self OK?
    grid.addLayout( vbox, 2, 1 )

  #// Create a slider

    sb = QSlider.new( 0, 300, 1, 100, QSlider::Horizontal,
			       self, "Slider" )
    sb.setTickmarks( QSlider::Below )
    sb.setTickInterval( 10 )
    sb.setFocusPolicy( QWidget::TabFocus )
    sb.setFixedHeight(sb.sizeHint.height)
    vbox.addWidget( sb )

    Qt::connect( sb, QSIGNAL("valueChanged(int)"), self, "sliderValueChanged" )
    QToolTip::add( sb, "slider" )

  #// Create a combo box

    combo = QComboBox.new( false, self, "comboBox" )
    combo.insertItem( "darkBlue" )
    combo.insertItem( "darkRed" )
    combo.insertItem( "darkGreen" )
    combo.insertItem( "blue" )
    combo.insertItem( "red" )
    combo.setFixedHeight(combo.sizeHint.height)
    vbox.addWidget( combo )
    Qt::connect( combo, QSIGNAL("activated(int)"), self, "comboBoxItemActivated" )
    QToolTip::add( combo, "read-only combo box" )

  #// Create an editable combo box

    edCombo = QComboBox.new( true, self, "edComboBox" )
    edCombo.insertItem( "Permutable" )
    edCombo.insertItem( "Malleable" )
    edCombo.insertItem( "Adaptable" )
    edCombo.insertItem( "Alterable" )
    edCombo.insertItem( "Inconstant" )
    edCombo.setFixedHeight(edCombo.sizeHint.height)
    vbox.addWidget( edCombo )
    Qt::connect( edCombo, QSIGNAL("activated(const char *)"),
	     self, "edComboBoxItemActivated" )
    QToolTip::add( edCombo, "editable combo box" )

    edCombo.setAutoCompletion( true )

    vbox.addStretch( 1 )

    vbox = QVBoxLayout.new(self, 8) #self OK?
    grid.addLayout( vbox, 2, 2 )

  #// Create a spin box

    spin = QSpinBox.new( 0, 10, 1, self, "spin" )
    spin.setSuffix(" mm")
    spin.setSpecialValueText( "Auto" )
    spin.setMinimumSizeQSize( spin.sizeHint )
    Qt::connect( spin, QSIGNAL( "valueChanged(const char*)" ),self, "spinBoxValueChanged" )
    QToolTip::add( spin, "spin box" )
    vbox.addWidget( spin )
    
    vbox.addStretch( 1 )

  #// Create a multi line edit

    mle = QMultiLineEdit.new( self, "multiLineEdit" )

    grid.addMultiCellWidget( mle, 3, 3, 1, 2 )
    mle.setMinimumHeight(mle.fontMetrics.height * 3 )
    mle.setText("This is a QMultiLineEdit widget,\n" + "useful for small multi-line\n" + "input fields.")
    QToolTip::add( mle, "multi line editor" )

  #// Create a single line edit

    le = QLineEdit.new( self, "lineEdit" )
    grid.addMultiCellWidget( le, 4, 4, 1, 2 )
    le.setFixedHeight(le.sizeHint.height)
    Qt::connect( le, QSIGNAL("textChanged(const char *)"),
	         self, "lineEditTextChanged" )
    QToolTip::add( le, "single line editor" )

  #// Create a horizontal line (sort of QFrame) above the message line

    separator = QFrame.new( self, "separatorLine" )
    separator.setFrameStyle( QFrame::HLine | QFrame::Sunken )
    separator.setFixedHeight( separator.sizeHint.height )
    grid.addMultiCellWidget( separator, 5, 5, 0, 2 )
    QToolTip::add( separator, "tool tips on a separator! wow!" )

    grid.setRowStretch(0,0)
    grid.setRowStretch(1,0)
    grid.setRowStretch(2,0)
    grid.setRowStretch(3,1)
    grid.setRowStretch(4,1)
    grid.setRowStretch(5,0)

    grid.setColStretch(0,1)
    grid.setColStretch(1,1)
    grid.setColStretch(2,1)


  #// Create an label and a message in a plain widget
  #// The message is updated when buttons are clicked etc.

    hbox = QHBoxLayout.new(self) #self OK?
    topLayout.addLayout( hbox )
    msgLabel = QLabel.new("", self, "msgLabel" )
    msgLabel.setText( "Message:" )
    msgLabel.setAlignment( AlignHCenter | AlignVCenter )
    msgLabel.setFixedSizeQSize( msgLabel.sizeHint )
    hbox.addWidget( msgLabel )
    QToolTip::add( msgLabel, "label 1" )

    @msg = QLabel.new("", self, "message" )
    @msg.setFrameStyle( QFrame::Panel | QFrame::Sunken )
    @msg.setAlignment( AlignCenter )
    @msg.setFont( QFont.new("times",12,QFont::Bold) )
    @msg.setText( "Message" )
    @msg.setFixedHeight( @msg.sizeHint.height )
    @msg.setText( "" )
    hbox.addWidget( @msg, 5 )
    QToolTip::add( @msg, "label 2" )


    topLayout.activate
end

def setStatus text
    @msg.setText(text);
end

def button1Clicked
    @msg.setText( "The first push button was clicked" )
end


def movieUpdate r  #( const QRect& )
    #// Uncomment this to test animated icons on your window manager
    setIcon( @movie.framePixmap )
end

def movieStatus s
    case s
      when QMovie::SourceEmpty
	@movielabel.setText("Could not load\n" + MOVIEFILENAME )
	@movielabel.setAlignment( AlignCenter )
	@movielabel.setBackgroundColor( backgroundColor )
      else
	if ( @movielabel.movie )	 	#// for flicker-free animation:
	   @movielabel.setBackgroundMode( NoBackground )
        end
    end
end


def checkBoxClicked id
    str = "Check box #{id} clicked : "
    chk = "---"
    if ( @cb[0].isChecked )
	chk[0] = 'r'
    end
    if ( @cb[1].isChecked )
	chk[1] = 'w'
    end
    if ( @cb[2].isChecked )
	chk[2] = 'x'
    end
    str += chk
    @msg.setText( str )
end


def edComboBoxItemActivated text
    str = "Editable Combo Box set to #{text}"
    @msg.setText( str )
end


def radioButtonClicked id
    str = "Radio button ##{id} clicked"
    @msg.setText( str )
end


def listBoxItemSelected index
    str = "List box item #{index} selected"
    @msg.setText( str )
end


def sliderValueChanged value
    str =  "Movie set to #{value}% of normal speed"
    @msg.setText( str )
    @movie.setSpeed( value )
end


def comboBoxItemActivated index
    str =  "Combo box item #{index} activated"
    @msg.setText( str )
    case index
    when 0
	QApplication::setWinStyleHighlightColor( QColor("dark blue") )
    when 1
	QApplication::setWinStyleHighlightColor( QColor("dark red") )
    when 2
	QApplication::setWinStyleHighlightColor( QColor("dark green") )
    when 3
	QApplication::setWinStyleHighlightColor( QColor("blue") )
    when 4
	QApplication::setWinStyleHighlightColor( QColor("red") )
    else
         QApplication::setWinStyleHighlightColor( QColor("dark blue") )
    end
end



def lineEditTextChanged newText
    str = "Line edit text: "
    str += newText
    @msg.setText( str )
end


def spinBoxValueChanged valueText
    str = "Spin box value: "
    str += valueText
    @msg.setText( str )
end

#//
#// All application events are passed throught this event filter.
#// We're using it to display some information about a clicked
#// widget (right mouse button + CTRL).
#//
=begin
bool WidgetView::eventFilter( QObject *obj, QEvent *event )
{
    static bool identify_now = TRUE;
    if ( event->type() == Event_MouseButtonPress && identify_now ) {
	QMouseEvent *e = (QMouseEvent*)event;
	if ( e->button() == RightButton && (e->state() & ControlButton) != 0 ){
	    QString str = "The clicked widget is a\n";
	    str += obj->className();
	    str += "\nThe widget's name is\n";
	    if ( obj->name() )
		str += obj->name();
	    else
		str += "<no name>";
	    identify_now = FALSE;		// don't do it in message box
	    QMessageBox::message( "Identify Widget", str, 0, (QWidget*)obj );
	    identify_now = TRUE;		// allow it again
	}
    }
    return FALSE;				// don't eat event
}
=end
#//
#// Include the meta-object code for classes in this file
#//
end
#include "widgets.moc"


#//
#// Create and display our WidgetView.
#//


    QApplication::setColorSpec( QApplication::CustomColor )
    QApplication::setStyle(WindowsStyle)
    a = QApplication.new( [$0] + ARGV)
    w = WidgetView.new
    a.setMainWidget( w )
    w.show()
    a.exec()

